@@ -8,11 +8,22 @@ from django import VERSION
 
 from .settings import api_settings
 
+
+try:
+    from django.urls import include
+except ImportError:
+    from django.conf.urls import include  # noqa: F401
+
+
+try:
+  from django.conf.urls import url
+except ImportError:
+  from django.urls import url
+
+
 try:
-    from django.urls import include, url
     from django.utils.translation import gettext as gettext_lazy
 except ImportError:
-    from django.conf.urls import include, url  # noqa: F401
     from django.utils.translation import ugettext as gettext_lazy
 
 
@@ -20,7 +31,8 @@ try:
     from django.utils.encoding import smart_str
 except ImportError:
     from django.utils.encoding import smart_text as smart_str
-    
+
+
 def has_set_cookie_samesite():
     return (VERSION >= (2,1,0))
 
